package org.apache.bcel.generic;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Stream;
import org.apache.bcel.classfile.AnnotationEntry;
import org.apache.bcel.classfile.Annotations;
import org.apache.bcel.classfile.Attribute;
import org.apache.bcel.classfile.Constant;
import org.apache.bcel.classfile.ConstantObject;
import org.apache.bcel.classfile.ConstantPool;
import org.apache.bcel.classfile.ConstantValue;
import org.apache.bcel.classfile.Field;
import org.apache.bcel.classfile.Utility;
import org.apache.bcel.generic.AnnotationEntryGen;
import org.apache.bcel.generic.ClassGenException;
import org.apache.bcel.generic.ConstantPoolGen;
import org.apache.bcel.generic.FieldGenOrMethodGen;
import org.apache.bcel.generic.FieldObserver;
import org.apache.bcel.generic.ObjectType;
import org.apache.bcel.generic.Type;
import org.apache.bcel.util.BCELComparator;

public class FieldGen extends FieldGenOrMethodGen {
  private static BCELComparator bcelComparator = (BCELComparator)new Object();
  
  private Object value;
  
  private List<FieldObserver> observers;
  
  public static BCELComparator getComparator() {
    return bcelComparator;
  }
  
  public static void setComparator(BCELComparator comparator) {
    bcelComparator = comparator;
  }
  
  public FieldGen(Field field, ConstantPoolGen cp) {
    this(field.getAccessFlags(), Type.getType(field.getSignature()), field.getName(), cp);
    Attribute[] attrs = field.getAttributes();
    for (Attribute attr : attrs) {
      if (attr instanceof ConstantValue) {
        setValue(((ConstantValue)attr).getConstantValueIndex());
      } else if (attr instanceof Annotations) {
        Annotations runtimeAnnotations = (Annotations)attr;
        runtimeAnnotations.forEach(element -> addAnnotationEntry(new AnnotationEntryGen(element, cp, false)));
      } else {
        addAttribute(attr);
      } 
    } 
  }
  
  public FieldGen(int accessFlags, Type type, String name, ConstantPoolGen cp) {
    super(accessFlags);
    setType(type);
    setName(name);
    setConstantPool(cp);
  }
  
  private void addAnnotationsAsAttribute(ConstantPoolGen cp) {
    Stream.<Attribute>of(AnnotationEntryGen.getAnnotationAttributes(cp, getAnnotationEntries())).forEach(this::addAttribute);
  }
  
  private int addConstant() {
    switch (getType().getType()) {
      case 4:
      case 5:
      case 8:
      case 9:
      case 10:
        return getConstantPool().addInteger(((Integer)this.value).intValue());
      case 6:
        return getConstantPool().addFloat(((Float)this.value).floatValue());
      case 7:
        return getConstantPool().addDouble(((Double)this.value).doubleValue());
      case 11:
        return getConstantPool().addLong(((Long)this.value).longValue());
      case 14:
        return getConstantPool().addString((String)this.value);
    } 
    throw new IllegalStateException("Unhandled : " + getType().getType());
  }
  
  public void addObserver(FieldObserver o) {
    if (this.observers == null)
      this.observers = new ArrayList<>(); 
    this.observers.add(o);
  }
  
  public void cancelInitValue() {
    this.value = null;
  }
  
  private void checkType(Type atype) {
    Type superType = getType();
    if (superType == null)
      throw new ClassGenException("You haven't defined the type of the field yet"); 
    if (!isFinal())
      throw new ClassGenException("Only final fields may have an initial value!"); 
    if (!superType.equals(atype))
      throw new ClassGenException("Types are not compatible: " + superType + " vs. " + atype); 
  }
  
  public org.apache.bcel.generic.FieldGen copy(ConstantPoolGen cp) {
    org.apache.bcel.generic.FieldGen fg = (org.apache.bcel.generic.FieldGen)clone();
    fg.setConstantPool(cp);
    return fg;
  }
  
  public boolean equals(Object obj) {
    return bcelComparator.equals(this, obj);
  }
  
  public Field getField() {
    String signature = getSignature();
    int nameIndex = getConstantPool().addUtf8(getName());
    int signatureIndex = getConstantPool().addUtf8(signature);
    if (this.value != null) {
      checkType(getType());
      int index = addConstant();
      addAttribute(new ConstantValue(getConstantPool().addUtf8("ConstantValue"), 2, index, getConstantPool().getConstantPool()));
    } 
    addAnnotationsAsAttribute(getConstantPool());
    return new Field(getAccessFlags(), nameIndex, signatureIndex, getAttributes(), getConstantPool().getConstantPool());
  }
  
  public String getInitValue() {
    if (this.value != null)
      return this.value.toString(); 
    return null;
  }
  
  public String getSignature() {
    return getType().getSignature();
  }
  
  public int hashCode() {
    return bcelComparator.hashCode(this);
  }
  
  public void removeObserver(FieldObserver o) {
    if (this.observers != null)
      this.observers.remove(o); 
  }
  
  public void setInitValue(boolean b) {
    checkType(Type.BOOLEAN);
    if (b)
      this.value = Integer.valueOf(1); 
  }
  
  public void setInitValue(byte b) {
    checkType(Type.BYTE);
    if (b != 0)
      this.value = Integer.valueOf(b); 
  }
  
  public void setInitValue(char c) {
    checkType(Type.CHAR);
    if (c != '\000')
      this.value = Integer.valueOf(c); 
  }
  
  public void setInitValue(double d) {
    checkType(Type.DOUBLE);
    if (d != 0.0D)
      this.value = Double.valueOf(d); 
  }
  
  public void setInitValue(float f) {
    checkType(Type.FLOAT);
    if (f != 0.0D)
      this.value = Float.valueOf(f); 
  }
  
  public void setInitValue(int i) {
    checkType(Type.INT);
    if (i != 0)
      this.value = Integer.valueOf(i); 
  }
  
  public void setInitValue(long l) {
    checkType(Type.LONG);
    if (l != 0L)
      this.value = Long.valueOf(l); 
  }
  
  public void setInitValue(short s) {
    checkType(Type.SHORT);
    if (s != 0)
      this.value = Integer.valueOf(s); 
  }
  
  public void setInitValue(String str) {
    checkType(ObjectType.getInstance("java.lang.String"));
    if (str != null)
      this.value = str; 
  }
  
  private void setValue(int index) {
    ConstantPool cp = getConstantPool().getConstantPool();
    Constant c = cp.getConstant(index);
    this.value = ((ConstantObject)c).getConstantValue(cp);
  }
  
  public final String toString() {
    String access = Utility.accessToString(getAccessFlags());
    access = access.isEmpty() ? "" : (access + " ");
    String signature = getType().toString();
    String name = getName();
    StringBuilder buf = new StringBuilder(32);
    buf.append(access).append(signature).append(" ").append(name);
    String value = getInitValue();
    if (value != null)
      buf.append(" = ").append(value); 
    return buf.toString();
  }
  
  public void update() {
    if (this.observers != null)
      for (FieldObserver observer : this.observers)
        observer.notify(this);  
  }
}
